

#include <Arduino.h>
#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>
#include <WiFiMulti.h>
#include <WiFiClientSecure.h>

#include <WebSocketsServer.h>
#include <ESPmDNS.h>

#include <FS.h>
#include "SPIFFS.h"
#include <ArduinoJson.h>

WebServer server(80);
WiFiMulti WiFiMulti;
WebSocketsServer webSocket = WebSocketsServer(81);

const char *ssid = "MiniRC";
const char *password = "12345678";

int Motor_Left_UP = D5;
int Motor_Left_DOWN = D6;
int Motor_Right_UP = D4;
int Motor_Right_DOWN = D3;


void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t length) {

  switch (type) {
    case WStype_DISCONNECTED:
      Serial.printf("[%u] Disconnected!\n", num);
      break;
    case WStype_CONNECTED: {
        IPAddress ip = webSocket.remoteIP(num);
        Serial.printf("[%u] Connected from %d.%d.%d.%d url: %s\n", num, ip[0], ip[1], ip[2], ip[3], payload);
        webSocket.sendTXT(num, "Connected");
      }
      break;
    case WStype_TEXT:
      Serial.printf("[%u] Data: %s\n", num, payload);
      String json = (char * )payload;
      StaticJsonDocument<200> doc;


      // Deserialize the JSON document
      DeserializationError error = deserializeJson(doc, json);

      // Test if parsing succeeds.
      if (error) {
        Serial.print(F("deserializeJson() failed: "));
        Serial.println(error.f_str());
        return;
      }

      // Fetch values.
      float x_value = doc["x"];
      float Y_value = doc["y"];
      float speed_value = doc["speed"];
      float Angle = doc["angle"];

      // Print values.
      Serial.println("x_value" + String(x_value));
      Serial.println("Y_value" + String(Y_value));
      Serial.println("speed_value" + String(speed_value));
      Serial.println(Angle);

      serializeJson(doc, Serial);

      //if on the origin, turn off all motors
      if (x_value == 0 && Y_value == 0) {

        analogWrite( Motor_Left_UP, 0);
        analogWrite( Motor_Left_DOWN,  0);

        analogWrite( Motor_Right_UP, 0);
        analogWrite( Motor_Right_DOWN, 0);

      }
      else  if (Angle <= 90 &&   Angle >= 0  )  { // if on the 1st quadrant


        //compute speed for each motor
        int speed1 = int( speed_value * Y_value);
        int speed2 = int( speed_value );


        Serial.println("----------1st quadrant speed1:" + String( speed1) + " speed2:" + String( speed2));
        analogWrite( Motor_Left_UP, 0);
        analogWrite( Motor_Left_DOWN, speed2);

        analogWrite( Motor_Right_UP, 0 );
        analogWrite( Motor_Right_DOWN, speed1);

      }
      else  if (Angle <= 180 &&   Angle >= 90 ) { // if on the 2nd quadrant

        int speed1 = -int( speed_value * Y_value);
        int speed2 = int( speed_value );


        Serial.println("----------3rd quadrant:" + String( speed1) + " speed2:" + String( speed2));


        //compute speed for each motor
        analogWrite( Motor_Left_UP, 0 );
        analogWrite( Motor_Left_DOWN,  speed1);


        analogWrite( Motor_Right_UP,  0);
        analogWrite( Motor_Right_DOWN, speed2 );
      } else  if (Angle <= 270 &&   Angle >= 180 ) { // if on the 3rd quadrant

        //compute speed for each motor
        int speed1 = -int( speed_value * Y_value);

        int speed2 = int( speed_value );
        Serial.println("----------3rd quadrant:" + String( speed1) + " speed2:" + String( speed2));


        analogWrite( Motor_Left_UP,   speed1);
        analogWrite( Motor_Left_DOWN, 0);

        analogWrite( Motor_Right_UP,   speed2);
        analogWrite( Motor_Right_DOWN, 0);


      }
      else if (Angle <= 360 &&   Angle >= 270 ) { // if on the 4th quadrant

        //compute speed for each motor
        int speed1 = -int( speed_value * Y_value);

        int speed2 = int( speed_value );

        Serial.println("----------4th quadrant:" + String( speed1) + " speed2:" + String( speed2));

        analogWrite( Motor_Left_UP, speed_value);
        analogWrite( Motor_Left_DOWN, 0 );

        analogWrite( Motor_Right_UP, speed_value * Y_value  );
        analogWrite( Motor_Right_DOWN, 0 );
      }
      else if (x_value == 0 ) { // if along the y axis

        Serial.println("----------in the y axis");
        if ( Angle == 90) {

          //compute speed for each motor
          int speed1 = int( speed_value * Y_value);
          int speed2 = int( speed_value );


          analogWrite( Motor_Left_UP, 0);
          analogWrite( Motor_Left_DOWN,  speed1);


          analogWrite( Motor_Right_UP, 0);
          analogWrite( Motor_Right_DOWN,  speed2 );
        } else  if ( Angle == 270) {

          //compute speed for each motor
          int speed1 = int( speed_value * Y_value);
          int speed2 = int( speed_value );


          analogWrite( Motor_Left_UP, speed1 );
          analogWrite( Motor_Left_DOWN, 0 );


          analogWrite( Motor_Right_UP,  speed2 );
          analogWrite( Motor_Right_DOWN, 0);
        } else {

          //Turn of all motors

          analogWrite( Motor_Left_UP, 0);
          analogWrite( Motor_Left_DOWN,  0);


          analogWrite( Motor_Right_UP, 0);
          analogWrite( Motor_Right_DOWN, 0);
        }

      }



      break;
  }
}


//Loads the html file
bool handleFileRead(String path) {
  Serial.println("handleFileRead: " + path);
  if (path.endsWith("/")) path += "index..html";


  if ( SPIFFS.exists(path)) {
    File file = SPIFFS.open(path, "r");
    size_t sent = server.streamFile(file, "text/html");
    file.close();
    return true;
  }
  return false;
}



void setup() {

  //Define the motors as output
  pinMode(Motor_Left_UP, OUTPUT);
  pinMode(Motor_Left_DOWN, OUTPUT);
  pinMode(Motor_Right_UP, OUTPUT);
  pinMode(Motor_Right_DOWN, OUTPUT);


  //prepare the serial for debugging
  Serial.begin(115200);
  Serial.print("\n");
  Serial.setDebugOutput(true);

  //Enable spiff to load the html file for controls
  SPIFFS.begin();

  //Enable as an access point
  Serial.print("Configuring access point...");
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid, password);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(myIP);


  //Begin the websocket for communication with the nunchuck/ html file
  webSocket.begin();
  webSocket.onEvent(webSocketEvent);
  Serial.println("WebSocket server started.");



  server.onNotFound([]() {
    if (!handleFileRead(server.uri()))
      server.send(404, "text/plain", "FileNotFound");
  });

  server.begin();
  Serial.println("HTTP server started");
  delay(2000);


  //turn off all motors
  analogWrite( Motor_Left_UP, 0);
  analogWrite( Motor_Left_DOWN,  0);


  analogWrite( Motor_Right_UP, 0);
  analogWrite( Motor_Right_DOWN, 0);
}

void loop(void) {
  server.handleClient();
  webSocket.loop();
}
